from django import template
from .. import get_option as get_option_source, get_label as get_label_source, get_text as get_text_source

register = template.Library()


@register.simple_tag(takes_context=True)
def get_option(context, key, as_var=None):
    return _get_qoption_value(get_option_source, context, key, as_var)


@register.simple_tag(takes_context=True)
def get_label(context, key, as_var=None):
    return _get_qoption_value(get_label_source, context, key, as_var)


@register.simple_tag(takes_context=True)
def get_text(context, key, title_var='title', as_var=None):
    title, text = _get_qoption_value(get_text_source, context, key, None)
    context[title_var] = title
    if as_var:
        context[as_var] = text
        return ''
    return text


@register.simple_tag(takes_context=True)
def get_text_title(context, key, text_var='text', as_var=None):
    title, text = _get_qoption_value(get_text_source, context, key, None)
    context[text_var] = text
    if as_var:
        context[as_var] = title
        return ''
    return title


def _get_qoption_value(function, context, key, as_var):
    try:
        ret = function(key)
    except:
        ret = ""

    if as_var:
        context[as_var] = ret
        return ''
    else:
        return ret



from models import OptionCache, Option, Label, Text


def get_option(key, fail_silently=True):
    return _get_qoption_value(Option, key, fail_silently)


def get_label(key, fail_silently=True):
    return _get_qoption_value(Label, key, fail_silently)


def get_text(key, fail_silently=True):
    return _get_qoption_value(Text, key, fail_silently)


def _get_qoption_value(model, key, fail_silently):
    cache_key = model.cache_mask.format(key)
    value = OptionCache.get(cache_key)
    if not value:
        try:
            opt = model.objects.get(key=key)
            if hasattr(opt, 'title'):
                value = [opt.title, opt.text]
            else:
                value = opt.value
        except model.DoesNotExist:
            if fail_silently:
                value = ''
            else:
                raise model.DoesNotExist('No record with key "{0}"'.format(key))
        except model.MultipleObjectsReturned:
            if fail_silently:
                opt = model.objects.filter(key=key)[0]
                if hasattr(opt, 'title'):
                    value = [opt.title, opt.text]
                else:
                    value = opt.value
            else:
                raise model.DoesNotExist('Returned more than one record with key "{0}"'.format(key))

        OptionCache.set(cache_key, value)
    return value



from django.contrib import admin
from .models import Option, Label, Text
from django.conf import settings


if 'modeltranslation' in settings.INSTALLED_APPS:
    from modeltranslation.admin import TranslationAdmin
    QOptionModelAdminTranslated = TranslationAdmin
else:
    QOptionModelAdminTranslated = admin.ModelAdmin

if 'tinymce' in settings.INSTALLED_APPS:
    from django.db import models
    from tinymce.widgets import AdminTinyMCE
    qoptions_formfield_overrides = {
        models.TextField: {'widget': AdminTinyMCE},
    }
else:
    qoptions_formfield_overrides = {}


class QOptionOptionAdmin(admin.ModelAdmin):
    list_display = ('key', 'value')


class QOptionLabelAdmin(QOptionModelAdminTranslated):
    list_display = ('key', 'value')


class QOptionTextAdmin(QOptionModelAdminTranslated):
    list_display = ('key', 'title')
    formfield_overrides = qoptions_formfield_overrides


admin.site.register(Option, QOptionOptionAdmin)
admin.site.register(Label,  QOptionLabelAdmin)
admin.site.register(Text,   QOptionTextAdmin)



from modeltranslation.translator import translator, TranslationOptions
from options.models import Label, Text


class LabelTranslationOptions(TranslationOptions):
    fields = ('value',)
translator.register(Label, LabelTranslationOptions)


class TextTranslationOptions(TranslationOptions):
    fields = ('text', 'title')
translator.register(Text,  TextTranslationOptions)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import get_language
from django.core.cache import cache


class OptionCache(object):
    langs_cache_key = 'qoptlangs'

    @staticmethod
    def _getkey(key, lang=None):
        if lang is None:
            lang = get_language()
        return "qopt_%s_%s" % (key, lang)

    @staticmethod
    def get(key):
        return cache.get(OptionCache._getkey(key), None)

    @staticmethod
    def set(key, value):
        lang = get_language()
        langs = cache.get(OptionCache.langs_cache_key, set())
        langs.add(lang)
        cache.set(OptionCache.langs_cache_key, langs)

        cache.set(OptionCache._getkey(key), value)

    @staticmethod
    def delete(key):
        cache.delete(OptionCache._getkey(key))

    @staticmethod
    def delete_all_langs(key):
        for lang in cache.get(OptionCache.langs_cache_key, set()):
            cache.delete(OptionCache._getkey(key, lang))


class Option(models.Model):
    """
    Options model
    """
    key = models.CharField(_('Key'), max_length=50, unique=True)
    value = models.CharField(_('Value'), max_length=256)

    cache_mask = 'qo_o_{0}'

    class Meta:
        verbose_name = _('option')
        verbose_name_plural = _('options')
        ordering = ['key']

    def __unicode__(self):
        return self.key

    def save(self, *args, **kwargs):
        super(Option, self).save(*args, **kwargs)
        try:
            OptionCache.delete_all_langs(Option.cache_mask.format(self.key))
        except KeyError:
            pass


class Label(models.Model):
    key = models.CharField(_('Key'), max_length=50, unique=True)
    value = models.CharField(_('Value'), max_length=256)

    cache_mask = 'qo_l_{0}'

    class Meta:
        verbose_name = _('label')
        verbose_name_plural = _('labels')
        ordering = ['key']

    def __unicode__(self):
        return self.key

    def save(self, *args, **kwargs):
        super(Label, self).save(*args, **kwargs)
        try:
            OptionCache.delete_all_langs(Label.cache_mask.format(self.key))
        except KeyError:
            pass


class Text(models.Model):
    key = models.CharField(_('Key'), max_length=50, unique=True)
    title = models.CharField(_('Title'), max_length=256, blank=True)
    text = models.TextField(_('Text'))

    cache_mask = 'qo_t_{0}'

    class Meta:
        verbose_name = _('text')
        verbose_name_plural = _('texts')
        ordering = ['key']

    def __unicode__(self):
        return self.key

    def save(self, *args, **kwargs):
        super(Text, self).save(*args, **kwargs)
        try:
            OptionCache.delete_all_langs(Text.cache_mask.format(self.key))
        except KeyError:
            pass
